// =============================================================================
//
//   UnrealMeshViewer
//   Mainform
//
// =============================================================================
//   Copyright  2004 by Sascha Willems - http://www.delphigl.de
//   visit the Delphi OpenGL Community - http://www.delphigl.com
// =============================================================================
//   Contents of this file are subject to the >>GNU Public License (GPL)<< which
//   can be obtained here : http://opensource.org/licenses/gpl-license.php
//   So only use this file if you fully unterstand that license!!!
// =============================================================================

unit UnrealMeshViewer_MainForm;

interface

uses
  Windows, Messages, Classes, Graphics, Controls, Forms,
  StdCtrls, dglOpenGL, ExtCtrls, ToolWin, ComCtrls,
  ImgList, Menus, glUnrealMesh, Dialogs;

type
   TMainForm = class(TForm)
    glPanel: TPanel;
    Timer1: TTimer;
    ToolBar1: TToolBar;
    ImageList1: TImageList;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton6: TToolButton;
    SequencePopup: TPopupMenu;
    OpenUCDialog: TOpenDialog;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormMouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
    procedure ToolButton3Click(Sender: TObject);
    procedure ToolButton1Click(Sender: TObject);
    private
      { Private-Deklarationen }
    public
     procedure OnSequencePopupClick(Sender : TObject);
    end;


   TDrawStyle = (dsTexturedAndLit, dsTextured, dsUnTexturedAndLit, dsUnTextured, dsWireFrame,
                 dsTexturedAndWireFrame);


var
  MainForm       : TMainForm;
  UnrealMesh     : TUnrealMesh;
  DC             : HDC;
  RC             : HGLRC;
  Zoom           : Single;
  WireFrame      : Byte;
  DrawStyle      : TDrawStyle = dsTexturedAndLit;

implementation

{$R *.dfm}

procedure TMainForm.OnSequencePopupClick(Sender : TObject);
begin
with Sender as TMenuItem do
 UnrealMesh.CurrSequence := MenuIndex;
UnrealMesh.CurrFrame    := UnrealMesh.Sequence[UnrealMesh.CurrSequence].StartFrame;
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
 i     : Integer;
 MItem : TMenuItem;
begin
InitOpenGL;
DC := GetDC(glPanel.Handle);
RC := CreateRenderingContext(DC, [opDoubleBuffered], 32, 24, 0, 0, 0, 0);
ActivateRenderingContext(DC, RC);
glEnable(GL_DEPTH_TEST);
glDepthFunc(GL_LEQUAL);
glClearColor(0.5, 0.5, 0.5, 0);
UnrealMesh := TUnrealMesh.Create(self);
Timer1.Enabled := True;
Zoom := -256;
SequencePopup.Items.Clear;
for i := 0 to UnrealMesh.SequenceName.Count-1 do
 begin
 MItem := TMenuItem.Create(self);
 MItem.Caption := UnrealMesh.SequenceName[i];
 MItem.OnClick := OnSequencePopupClick;
 SequencePopup.Items.Add(MItem);
 end;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
DeactivateRenderingContext;
DestroyRenderingContext(RC);
ReleaseDC(glPanel.Handle, DC);
end;

procedure TMainForm.Timer1Timer(Sender: TObject);
const
 LightPos : array[0..3] of Single = (0, -40, -80, 1);
begin
glMatrixMode(GL_PROJECTION);
glLoadIdentity;
glViewport(0, 0, glPanel.Width, glPanel.Height);
gluPerspective(60, glPanel.Width/glPanel.Height, 1, 1024);
glMatrixMode(GL_MODELVIEW);
glLoadIdentity;
glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
glTranslatef(0, 0, Zoom);

case DrawStyle of
 dsTexturedAndLit :
  begin
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glLightfv(GL_LIGHT0, GL_POSITION, @LightPos);
  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D, UnrealMesh.Texture);
  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
  end;
 dsTextured :
  begin
  glDisable(GL_LIGHTING);
  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D, UnrealMesh.Texture);
  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
  end;
 dsUnTexturedAndLit :
  begin
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glLightfv(GL_LIGHT0, GL_POSITION, @LightPos);
  glDisable(GL_TEXTURE_2D);
  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
  end;
 dsUnTextured :
  begin
  glDisable(GL_TEXTURE_2D);
  glDisable(GL_LIGHTING);
  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
  end;
 dsWireFrame :
  begin
  glDisable(GL_LIGHTING);
  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D, UnrealMesh.Texture);
  glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
  end;
 dsTexturedAndWireFrame :
  begin
  glDisable(GL_LIGHTING);
  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D, UnrealMesh.Texture);
  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
  end;
end;

glColor3f(1, 1, 1);
if WireFrame = 1 then
 begin
 glDisable(GL_TEXTURE_2D);
 glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
 glColor3f(0, 0, 0);
 end;
UnrealMesh.Render;
if DrawStyle = dsTexturedAndWireFrame then
 begin
 glEnable(GL_POLYGON_OFFSET_FILL);
 glPolygonOffset(8, 4);
 glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
 glDisable(GL_TEXTURE_2D);
 UnrealMesh.Render;
 glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
 glDisable(GL_POLYGON_OFFSET_FILL);
 end;

SwapBuffers(DC);

UnrealMesh.ProgressSequence(1);
end;

procedure TMainForm.FormMouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
begin
Zoom := Zoom + WheelDelta * 0.1;
end;

procedure TMainForm.ToolButton3Click(Sender: TObject);
begin
if DrawStyle = dsTexturedAndWireFrame then
 DrawStyle := dsTexturedAndLit
else
 inc(DrawStyle);
end;

procedure TMainForm.ToolButton1Click(Sender: TObject);
var
 Prefix : String;
 MItem  : TMenuItem;
 i      : Integer;
begin
if OpenUCDialog.Execute then
 begin
 Prefix := Copy(OpenUCDialog.FileName, 1, Length(OpenUCDialog.FileName)-3);
 UnrealMesh.LoadFromFile(Prefix+'_d.3d', Prefix+'_a.3d', Prefix+'.uc', Prefix+'.jpg');
 SequencePopup.Items.Clear;
 for i := 0 to UnrealMesh.SequenceName.Count-1 do
  begin
  MItem := TMenuItem.Create(self);
  MItem.Caption := UnrealMesh.SequenceName[i];
  MItem.OnClick := OnSequencePopupClick;
  SequencePopup.Items.Add(MItem);
  end;
 end;
end;

end.
